window.Gleam.initSearch([{"doc":"gleojson","title":"gleojson","content":"# gleojson\n\n[![Package Version](https://img.shields.io/hexpm/v/gleojson)](https://hex.pm/packages/gleojson)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/gleojson/)\n\n**gleojson** is a comprehensive GeoJSON parsing and encoding library for Gleam, following the [RFC 7946](https://tools.ietf.org/html/rfc7946) specification. It provides robust types and utility functions to seamlessly encode and decode GeoJSON objects such as Points, LineStrings, Polygons, and more.\n\n**Note:** This package is currently in development and has not reached version 1.0.0 yet. The API is considered unstable and may undergo breaking changes in future releases. Please use with caution in production environments and expect potential updates that might require code changes.\n\n## Features\n\n- Full support for all GeoJSON object types: Point, MultiPoint, LineString, MultiLineString, Polygon, MultiPolygon, GeometryCollection, Feature, and FeatureCollection\n- Flexible encoding and decoding of GeoJSON objects\n- Custom property support for Feature and FeatureCollection objects\n- Type-safe representation of GeoJSON structures\n\n## Current Limitations\n\nWhile **gleojson** aims to fully implement the GeoJSON specification (RFC 7946), some features are still under development. Key areas for future improvement include:\n\n1. Coordinate validation\n1. Antimeridian and pole handling\n1. Bounding box support\n1. Right-hand rule enforcement for polygon orientation\n1. Position array length limitation\n1. GeometryCollection usage recommendations\n\nDespite these limitations, **gleojson** is fully functional for most common GeoJSON use cases.\n\n## Installation\n\nAdd **gleojson** to your Gleam project:\n\n```sh\ngleam add gleojson\n```\n\n## Usage\n\nHere&#39;s a basic example of how to use gleojson:\n\n```gleam\nimport gleojson\nimport gleam/json\nimport gleam/option\nimport gleam/io\n\npub fn main() {\n  // Create a Point geometry\n  let point = gleojson.Point([125.6, 10.1])\n\n  // Create a Feature with the Point geometry\n  let feature = gleojson.Feature(\n    geometry: option.Some(point),\n    properties: option.None,\n    id: option.Some(gleojson.StringId(&quot;example-point&quot;))\n  )\n\n  // Encode the Feature to GeoJSON\n  let geojson = gleojson.GeoJSONFeature(feature)\n  let encoded = gleojson.encode_geojson(geojson, gleojson.properties_null_encoder)\n\n  // Print the encoded GeoJSON\n  io.println(json.to_string(encoded))\n}\n```\n\nFor more advanced usage, including custom properties and decoding, see the [documentation](https://hexdocs.pm/gleojson).\n\n## Development\n\nTo build and test the project:\n\n```sh\ngleam build\ngleam test\n```\n\n## Contributing\n\nContributions to gleojson are welcome! Please feel free to submit a Pull Request. Before contributing, please review our [contribution guidelines](CONTRIBUTING.md).\n\n## License\n\nThis project is licensed under the MIT License. See the [LICENSE](LICENSE) file for details.\n\nPlease see the [NOTICE](NOTICE) file for information about third party components and the use of AI assistance in this project.\n","url":"index.html"},{"doc":"gleojson","title":"Feature","content":"pub type Feature(properties) {\n  Feature(\n    geometry: option.Option(Geometry),\n    properties: option.Option(properties),\n    id: option.Option(FeatureId),\n  )\n}\n\nFeature(\n  geometry: option.Option(Geometry),\n  properties: option.Option(properties),\n  id: option.Option(FeatureId),\n)\n\n\nSynonyms:\ngleojson.Feature\ngleojson Feature","url":"gleojson.html#Feature"},{"doc":"gleojson","title":"FeatureCollection","content":"pub type FeatureCollection(properties) {\n  FeatureCollection(features: List(Feature(properties)))\n}\n\nFeatureCollection(features: List(Feature(properties)))\n\n\nSynonyms:\ngleojson.FeatureCollection\ngleojson FeatureCollection","url":"gleojson.html#FeatureCollection"},{"doc":"gleojson","title":"FeatureId","content":"pub type FeatureId {\n  StringId(String)\n  NumberId(Float)\n}\n\nStringId(String)\n\n\nNumberId(Float)\n\n\nSynonyms:\ngleojson.FeatureId\ngleojson FeatureId","url":"gleojson.html#FeatureId"},{"doc":"gleojson","title":"GeoJSON","content":"pub type GeoJSON(properties) {\n  GeoJSONGeometry(Geometry)\n  GeoJSONFeature(Feature(properties))\n  GeoJSONFeatureCollection(FeatureCollection(properties))\n}\n\nGeoJSONGeometry(Geometry)\n\n\nGeoJSONFeature(Feature(properties))\n\n\nGeoJSONFeatureCollection(FeatureCollection(properties))\n\n\nSynonyms:\ngleojson.GeoJSON\ngleojson GeoJSON","url":"gleojson.html#GeoJSON"},{"doc":"gleojson","title":"Geometry","content":"pub type Geometry {\n  Point(coordinates: Position)\n  MultiPoint(coordinates: List(Position))\n  LineString(coordinates: List(Position))\n  MultiLineString(coordinates: List(List(Position)))\n  Polygon(coordinates: List(List(Position)))\n  MultiPolygon(coordinates: List(List(List(Position))))\n  GeometryCollection(geometries: List(Geometry))\n}\n\nPoint(coordinates: Position)\n\n\nMultiPoint(coordinates: List(Position))\n\n\nLineString(coordinates: List(Position))\n\n\nMultiLineString(coordinates: List(List(Position)))\n\n\nPolygon(coordinates: List(List(Position)))\n\n\nMultiPolygon(coordinates: List(List(List(Position))))\n\n\nGeometryCollection(geometries: List(Geometry))\n\n\nSynonyms:\ngleojson.Geometry\ngleojson Geometry","url":"gleojson.html#Geometry"},{"doc":"gleojson","title":"Position","content":"pub type Position =\n  List(Float)\n\n\nSynonyms:\ngleojson.Position\ngleojson Position","url":"gleojson.html#Position"},{"doc":"gleojson","title":"encode_geojson","content":"pub fn encode_geojson(\n  geojson: GeoJSON(a),\n  properties_encoder: fn(a) -&gt; Json,\n) -&gt; Json\n Encodes a GeoJSON object into a JSON value.\n\n This function takes a GeoJSON object and a properties encoder function,\n and returns a JSON representation of the GeoJSON object.\n\n ## Arguments\n\n - `geojson`: The GeoJSON object to encode.\n - `properties_encoder`: A function that encodes the properties of Features and FeatureCollections.\n\n ## Returns\n\n A JSON representation of the GeoJSON object.\n\n ## Example\n\n \n import gleojson\n import gleam/json\n import gleam/option\n import gleam/io\n\n pub type CustomProperties {\n   CustomProperties(name: String, value: Float)\n }\n\n pub fn custom_properties_encoder(props: CustomProperties) -&gt; json.Json {\n   json.object([\n     #(&quot;name&quot;, json.string(props.name)),\n     #(&quot;value&quot;, json.float(props.value)),\n   ])\n }\n\n pub fn main() {\n   let point = gleojson.Point([0.0, 0.0])\n   let properties = CustomProperties(&quot;Example&quot;, 42.0)\n   let feature = gleojson.Feature(\n     geometry: option.Some(point),\n     properties: option.Some(properties),\n     id: option.Some(gleojson.StringId(&quot;example-point&quot;))\n   )\n   let geojson = gleojson.GeoJSONFeature(feature)\n   \n   let encoded = gleojson.encode_geojson(geojson, custom_properties_encoder)\n   io.println(json.to_string(encoded))\n }\n \nSynonyms:\ngleojson.encode_geojson\ngleojson encode_geojson","url":"gleojson.html#encode_geojson"},{"doc":"gleojson","title":"geojson_decoder","content":"pub fn geojson_decoder(\n  properties_decoder: fn(Dynamic) -&gt; Result(a, List(DecodeError)),\n) -&gt; fn(Dynamic) -&gt; Result(GeoJSON(a), List(DecodeError))\n Decodes a GeoJSON object from a dynamic value.\n\n This function takes a dynamic value (typically parsed from JSON) and a properties decoder,\n and attempts to decode it into a GeoJSON object.\n\n ## Arguments\n\n - `properties_decoder`: A function that decodes the properties of Features and FeatureCollections.\n\n ## Returns\n\n A function that takes a dynamic value and returns a Result containing either\n the decoded GeoJSON object or a list of decode errors.\n\n ## Example\n\n \n import gleojson\n import gleam/json\n import gleam/result\n import gleam/dynamic\n import gleam/io\n import gleam/string\n\n pub type CustomProperties {\n   CustomProperties(name: String, value: Float)\n }\n\n pub fn custom_properties_decoder(\n   dyn: dynamic.Dynamic,\n ) -&gt; Result(CustomProperties, List(dynamic.DecodeError)) {\n   dynamic.decode2(\n     CustomProperties,\n     dynamic.field(&quot;name&quot;, dynamic.string),\n     dynamic.field(&quot;value&quot;, dynamic.float),\n   )(dyn)\n }\n\n pub fn main() {\n   let json_string = &quot;{\\&quot;type\\&quot;:\\&quot;Feature\\&quot;,\\&quot;geometry\\&quot;:{\\&quot;type\\&quot;:\\&quot;Point\\&quot;,\\&quot;coordinates\\&quot;:[0.0,0.0]},\\&quot;properties\\&quot;:{\\&quot;name\\&quot;:\\&quot;Example\\&quot;,\\&quot;value\\&quot;:42.0}}&quot;\n   \n   let decoded = \n     json.decode(\n       from: json_string,\n       using: gleojson.geojson_decoder(custom_properties_decoder)\n     )\n   \n   case decoded {\n     Ok(geojson) -&gt; {\n       // Work with the decoded GeoJSON object\n       case geojson {\n         gleojson.GeoJSONFeature(feature) -&gt; {\n           io.println(&quot;Decoded a feature&quot;)\n         }\n         _ -&gt; io.println(&quot;Decoded a different type of GeoJSON object&quot;)\n       }\n     }\n     Error(errors) -&gt; {\n       // Handle decoding errors\n       io.println(&quot;Failed to decode: &quot; &lt;&gt; string.join(errors, &quot;, &quot;))\n     }\n   }\n }\n \n\n Note: This function expects a valid GeoJSON structure. Invalid or incomplete\n GeoJSON data will result in a decode error.\nSynonyms:\ngleojson.geojson_decoder\ngleojson geojson_decoder","url":"gleojson.html#geojson_decoder"},{"doc":"gleojson","title":"properties_null_decoder","content":"pub fn properties_null_decoder(\n  dyn: a,\n) -&gt; Result(Nil, List(DecodeError))\n Decodes null properties for Features and FeatureCollections.\n\n This is a utility function that can be used as the `properties_decoder`\n argument for `geojson_decoder` when you don&#39;t need to decode any properties.\n\n ## Returns\n\n Always returns `Ok(Nil)`.\n\n ## Example\n\n \n import gleojson\n import gleam/json\n import gleam/result\n\n pub fn main() {\n   let json_string = &quot;{\\&quot;type\\&quot;:\\&quot;Feature\\&quot;,\\&quot;geometry\\&quot;:{\\&quot;type\\&quot;:\\&quot;Point\\&quot;,\\&quot;coordinates\\&quot;:[0.0,0.0]},\\&quot;properties\\&quot;:null}&quot;\n   \n   let decoded = \n     json.decode(\n       from: json_string,\n       using: gleojson.geojson_decoder(gleojson.properties_null_decoder)\n     )\n   \n   // The &quot;properties&quot; field in the decoded Feature will be None\n }\n \nSynonyms:\ngleojson.properties_null_decoder\ngleojson properties_null_decoder","url":"gleojson.html#properties_null_decoder"},{"doc":"gleojson","title":"properties_null_encoder","content":"pub fn properties_null_encoder(props: a) -&gt; Json\n Encodes null properties for Features and FeatureCollections.\n\n This is a utility function that can be used as the `properties_encoder`\n argument for `encode_geojson` when you don&#39;t need to encode any properties.\n\n ## Returns\n\n A JSON null value.\n\n ## Example\n\n \n import gleojson\n import gleam/json\n import gleam/option\n\n pub fn main() {\n   let point = gleojson.Point([0.0, 0.0])\n   let feature = gleojson.Feature(\n     geometry: option.Some(point),\n     properties: option.None,\n     id: option.None\n   )\n   let geojson = gleojson.GeoJSONFeature(feature)\n   \n   let encoded = gleojson.encode_geojson(geojson, gleojson.properties_null_encoder)\n   // The &quot;properties&quot; field in the resulting JSON will be null\n }\n \nSynonyms:\ngleojson.properties_null_encoder\ngleojson properties_null_encoder","url":"gleojson.html#properties_null_encoder"},{"doc":"gleojson","title":"gleojson","content":" Functions for working with GeoJSON data.\n\n This module provides types and functions for encoding and decoding GeoJSON data.\n It supports all GeoJSON object types including Point, MultiPoint, LineString,\n MultiLineString, Polygon, MultiPolygon, GeometryCollection, Feature, and FeatureCollection.\n\n ## Usage\n\n To use this module, you can import it in your Gleam code:\n\n ```gleam\n import gleojson\n ```\n\n Then you can use the provided functions to encode and decode GeoJSON data.","url":"gleojson.html"}]);